/*****************************************************************************
* Copyright (c) Bridgetek Pte Ltd.
* Software License Agreement
*
* This code is provided as an example only and is not guaranteed by Bridgetek. 
* BRIDGETEK accept no responsibility for any issues resulting from its use. 
* The developer of the final application incorporating any parts of this 
* sample project is responsible for ensuring its safe and correct operation 
* and for any consequences resulting from its use.
*****************************************************************************/
/**
* @file  App_MeterDemo.c/.ino
* @brief Sample application to demonstrate EVE primitives and built-in widgets 
Versoin 5.0 - April/17/2017 - Restructure source code
version 2.1 - Jun/13/2014 - Scrolling velocity changes for FT801 platform. 
Version 2.0 - Support for FT801 platform.
Version 1.0 - Final version based on the requirements.
Version 0.1 - intial draft of the release notes

*
*/


#include "Platform.h"
#include "App_Common.h"
#include "App_MeterDemo.h"

#define Absolute_Dial
//#define Relative_Dial

#define STARTUP_ADDRESS	100*1024L
#define ICON_ADDRESS	250*1024L
#define BLK_LEN	  1024

#define CLOCK_HANDLE	(0x01)

#define DIAL		1


int32_t BaseTrackVal = 0,BaseTrackValInit = 0,BaseTrackValSign = 0,MemLoc=0;
int16_t MoveDeg =0;
float_t angle;
/* Global used for Hal Context */
Gpu_Hal_Context_t host,*phost;


#ifdef Relative_Dial
static struct {
    int32_t dragprev;
    int32_t vel;      // velocity
    int32_t base;    // screen x coordinate, in 1/16ths pixel
    //  int32_t limit;
}scroller ;

static void scroller_init(uint32_t limit)
{
    scroller.dragprev = 0;
    scroller.vel = 0;      // velocity
    scroller.base = 0;     // screen x coordinate, in 1/16ths pixel
    // scroller.limit = limit;
}

static void scroller_run()
{
    int32_t change;
    uint32_t sx;
    int32_t sxtemp=0,sxprev=0,dragdeg=0;

    Gpu_CoCmd_Track(phost,(240)+40,136, DIAL, DIAL, 1);
    //  signed short sx = Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY + 2);

    sx =   Gpu_Hal_Rd32(phost,REG_TRACKER);
    // printf("%d\n,",sx);


    /*if ((sx != -32768) & (scroller.dragprev != -32768)) */
    if ((sx != 0) && (scroller.dragprev !=0)) 
    {
        scroller.vel = (scroller.dragprev - sx) << 4;
    } 
    else
    {
        change = MAX(1, abs(scroller.vel) >> 5);
        if (scroller.vel < 0)
        scroller.vel += change;
        if (scroller.vel > 0)
        scroller.vel -= change;
    }
    scroller.dragprev = sx;
    // printf("%d\n,sx", sx);
    scroller.base += scroller.vel;
    scroller.base = MAX(0, (scroller.base));

}
#endif



void Load_RawDataFromfile(prog_uchar8_t ImageArrayname[],/* Image Array*/

uint32_t ptr,/* Array Size*/
//uint32_t RamAddr,
int16_t Handle,
int16_t Format,
int16_t Stride,
int16_t Width,
int16_t Height								 
)
{


    if(Handle== 0)
    {

        App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));
        App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
        App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(0));
        App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT(L4,146/2,146));		
        App_WrCoCmd_Buffer(phost, BITMAP_SIZE(BILINEAR,BORDER,BORDER,146,146));
        Gpu_Hal_WrCmd32(phost,  CMD_INFLATE);
        Gpu_Hal_WrCmd32(phost,  0/*RamAddr*/);
#if  defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
        Gpu_Hal_WrCmdBuf(phost,SAMAPP_Abs_Base_Dial_L1,SAMAPP_Abs_BaseDial_Size146x146_L1);
#endif
#if defined(ARDUINO_PLATFORM) || defined(FT9XX_PLATFORM)
        Gpu_Hal_WrCmdBufFromFlash(phost,ImageArrayname,ptr);
#endif
        App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
        App_WrCoCmd_Buffer(phost, VERTEX2II(0,0,0,0));		
    }
    else if(Handle== 1)
    {
        Gpu_Hal_WrCmd32(phost,  CMD_INFLATE);
        Gpu_Hal_WrCmd32(phost,  15000/*RamAddr*/);

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
        Gpu_Hal_WrCmdBuf(phost,ImageArrayname,ptr);
#endif
#if defined(ARDUINO_PLATFORM) || defined(FT9XX_PLATFORM)
        Gpu_Hal_WrCmdBufFromFlash(phost,ImageArrayname,ptr);
#endif
        App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(Handle));
        App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(15000));
        App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT(Format,Stride,Height));		
        App_WrCoCmd_Buffer(phost, BITMAP_SIZE(NEAREST,BORDER,BORDER,Width,Height));
#ifdef DISPLAY_RESOLUTION_WVGA
        App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT_H(Stride>>10,Height>>9));
        App_WrCoCmd_Buffer(phost, BITMAP_SIZE_H(Width>>9,Height>>9));
#endif
        //App_WrCoCmd_Buffer(phost, VERTEX2II(150,0,Handle,0));		
    }		

}

#ifdef Absolute_Dial
void Abs_Meter_Dial()
{
    uint8_t i=0,Volume=0,Read_tag,prevtag=0;
    int16_t ScreenWidthMid,ScreenHeightMid,th,theta,Touch,NoTouch,Sx1=0,Sy1=0;
    int16_t OuterCircleRadius,PrevVal=0,CircleX,CircleY,ButY,ButWid,NoY,Tx1,Tx2,Tx3,Ty1,Ty2,Ty3 ;

    int32_t PrevTh,CurrTh,adjusting=0,BaseTrackValSign = 0,PrevBaseVal=0,Delta =0,X2,Y2,X1,Y1;

    uint32_t RotaryTag=0,G1,G2;


    ScreenWidthMid = DispWidth/2;
    ScreenHeightMid = DispHeight/2;

    NoTouch = -32768;

    MoveDeg = 31;

    Tx1= Tx2=Tx3 =Ty1=Ty2=Ty3=0;

#if defined(DISPLAY_RESOLUTION_WQVGA) 
    Sx1 = Sy1 = DispWidth;
    OuterCircleRadius = ScreenWidthMid - 144;
    CircleX = ScreenWidthMid;
    CircleY = ScreenHeightMid;
    ButY = DispHeight -262;
    ButWid = 27;
    NoY = (DispHeight-262 + 30/2);
    Tx1 = ScreenWidthMid-86;
    Tx2 = ScreenWidthMid-70 + 146/2;
    Ty2 = ScreenHeightMid-67 + 146/2;
    Tx3 = ScreenWidthMid+72;

#elif defined(DISPLAY_RESOLUTION_QVGA)
    Sx1 = Sy1 = DispWidth;
    ButY = 1;
    ButWid = 23;
    NoY = 12;
    OuterCircleRadius = ScreenWidthMid - 64;
    CircleX = ScreenWidthMid-70 + 146/2;
    CircleY = ScreenHeightMid-67 + 146/2;
    Tx1 = ScreenWidthMid-63;
    Tx2 = ScreenWidthMid-70 + 146/2;
    Ty2 = ScreenHeightMid-67 + 146/2;
    Tx3 = ScreenWidthMid+72;
#elif defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
    Sx1 = Sy1 = DispWidth;
	ButY = 1;
	ButWid = 27;
	NoY = 12;
	OuterCircleRadius = ScreenWidthMid - 64;
	CircleX = ScreenWidthMid-70 + 146/2;
	CircleY = ScreenHeightMid-67 + 146/2;
	Tx1 = ScreenWidthMid-130;
	Tx2 = ScreenWidthMid-70 + 146/2;
	Ty2 = ScreenHeightMid-67 + 146/2;
	Tx3 = ScreenWidthMid+140;
#elif defined(DISPLAY_RESOLUTION_WVGA)
    Sx1 = Sy1 = DispWidth;
    OuterCircleRadius = ScreenWidthMid - 250;
    CircleX = ScreenWidthMid;
    CircleY = ScreenHeightMid;
    ButY = DispHeight - 462;
    ButWid = 45;
    NoY = (DispHeight-462 + 42/2);
    Tx1 = ScreenWidthMid-138;
    Tx2 = ScreenWidthMid-130 + 253/2; //ScreenWidthMid-130 + 243/2;
    Ty2 = ScreenHeightMid-117 + 243/2;
    Tx3 = ScreenWidthMid+152; //ScreenWidthMid+148;

#else
    Sx1 = Sy1 = DispWidth;
    OuterCircleRadius = ScreenWidthMid - 144;
    CircleX = ScreenWidthMid;
    CircleY = ScreenHeightMid;
    ButY = DispHeight -262;
    ButWid = 27;
    NoY = (DispHeight-262 + 30/2);
    Tx1 = ScreenWidthMid-86;
    Tx2 = ScreenWidthMid-70 + 146/2;
    Ty2 = ScreenHeightMid-67 + 146/2;
    Tx3 = ScreenWidthMid+72;

#endif



    Gpu_Hal_WrCmd32(phost, CMD_MEMSET);
    Gpu_Hal_WrCmd32(phost, 0L);//starting address of memset
    Gpu_Hal_WrCmd32(phost, 55L);//value of memset
    Gpu_Hal_WrCmd32(phost, 256*1024);



    /* set the vol to MAX */
    Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0);
    /* Play carousel*/
    Gpu_Hal_Wr16(phost, REG_SOUND, 8);

    /* Construction of starting screen shot, assign all the bitmap handles here */
    App_WrCoCmd_Buffer(phost,CMD_DLSTART);
    App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));

    App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

    /* Download the raw data into intended locations */
    {
        App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));	

        Load_RawDataFromfile(SAMAPP_Abs_Base_Dial_L1,SAMAPP_Abs_BaseDial_Size146x146_L1,0,L4,146,146,146);
        App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
        Load_RawDataFromfile(SAMAPP_Abs_Meter_Dial_rgb,SAMAPP_Abs_MeterDial_Size146x146_rgb,1,RGB565,146*2,146,146);	

    }

    App_WrCoCmd_Buffer(phost,DISPLAY());
    Gpu_CoCmd_Swap(phost);

    /* Download the commands into fifo */
    App_Flush_Co_Buffer(phost);

    /* Wait till coprocessor completes the operation */
    Gpu_Hal_WaitCmdfifo_empty(phost);


    /*asiign track to the black dot*/
    Gpu_CoCmd_Track(phost,CircleX,CircleY, DIAL, DIAL, 1);
    G1 = 0x1E1E1E;	
    G2 = 0x999999;

    Gpu_Hal_Wr8(phost, REG_PLAY,1);

    do{
        Read_tag = Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
        Touch = Gpu_Hal_Rd16(phost, REG_TOUCH_SCREEN_XY+2);
        RotaryTag = Gpu_Hal_Rd32(phost,REG_TRACKER);
        Gpu_CoCmd_Dlstart(phost);
        App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
        App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));

        App_WrCoCmd_Buffer(phost, SCISSOR_XY(0,0) );

        Gpu_CoCmd_Gradient(phost,0, 0, G1,Sx1, Sy1, G2);



        App_WrCoCmd_Buffer(phost, SCISSOR_SIZE(Sx1,Sy1) );	

        //App_WrCoCmd_Buffer(phost, SCISSOR_SIZE(512,512) );
        App_WrCoCmd_Buffer(phost, SCISSOR_SIZE(DispWidth,DispHeight) );

        /* Assign Stencil Value to get the triangle shape */
        App_WrCoCmd_Buffer(phost, COLOR_MASK(0,0,0,1) );//only alpha is enabled,so that no color is used for strip
        App_WrCoCmd_Buffer(phost, LINE_WIDTH(1*16));
        App_WrCoCmd_Buffer(phost, STENCIL_OP(INCR,INVERT) );
        App_WrCoCmd_Buffer(phost, STENCIL_FUNC(EQUAL,0,255) );
        App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_B));		
        App_WrCoCmd_Buffer(phost, COLOR_RGB(136 ,136 ,136)); 


        //App_WrCoCmd_Buffer(phost, VERTEX2II(Tx1,DispHeight,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(Tx1*16,DispHeight*16));
        //App_WrCoCmd_Buffer(phost, VERTEX2II(Tx2,Ty2,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(Tx2*16,Ty2*16));
        //App_WrCoCmd_Buffer(phost, VERTEX2II(Tx3,DispHeight,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(Tx3*16,DispHeight*16));

        App_WrCoCmd_Buffer(phost, STENCIL_OP(KEEP,INCR) );
        App_WrCoCmd_Buffer(phost, STENCIL_FUNC(NOTEQUAL,255,255) );
        App_WrCoCmd_Buffer(phost, COLOR_MASK(1,1,1,1) );//enable all colors


        /* Outer Black circle*/
        /* Based on stencil update Points color,leaving the triangular area*/		

        App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
        App_WrCoCmd_Buffer(phost,POINT_SIZE((OuterCircleRadius) * 16) );
        App_WrCoCmd_Buffer(phost,COLOR_RGB(0 ,0 ,0)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE(((OuterCircleRadius-3) * 16)) );		
        App_WrCoCmd_Buffer(phost,COLOR_RGB (15,77,116)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE(((OuterCircleRadius-5) * 16)) );		
        App_WrCoCmd_Buffer(phost,COLOR_RGB (54,115,162)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE(((OuterCircleRadius-7) * 16)) );		
        App_WrCoCmd_Buffer(phost,COLOR_RGB (76,195,225)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE(((OuterCircleRadius-9) * 16)) );		
        App_WrCoCmd_Buffer(phost,COLOR_RGB (89,211,232)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE(((OuterCircleRadius-11) * 16)) );		
        App_WrCoCmd_Buffer(phost,COLOR_RGB (123,248,250)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE((OuterCircleRadius-17) * 16) );		
        App_WrCoCmd_Buffer(phost,COLOR_RGB (89,211,232)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE((OuterCircleRadius-18) * 16) );
        App_WrCoCmd_Buffer(phost,COLOR_RGB (76,195,225)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE((OuterCircleRadius-19) * 16) );
        App_WrCoCmd_Buffer(phost,COLOR_RGB (54,115,162)); 
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));

        App_WrCoCmd_Buffer(phost,POINT_SIZE((OuterCircleRadius-20) * 16) );	
        App_WrCoCmd_Buffer(phost,COLOR_RGB (15,77,116));  
        //App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));  
        App_WrCoCmd_Buffer(phost, VERTEX2F(CircleX*16,CircleY*16));



        /*Managing the track*/	
        if((Touch != NoTouch))
        {
            if((RotaryTag & 0xff)== 1)
            {
                CurrTh = (int16_t)((RotaryTag >> 16) & 0xffff);
                /* BaseTrackVal is the rotary track value of the particular white circle that is moving */
                if((adjusting!=0) )
                {					
                    BaseTrackValSign += (int16_t)(CurrTh - PrevTh);
                } 
                PrevTh = CurrTh;
                adjusting = (RotaryTag & 0xff);				
                //PrevValDeg = NextTh;

            }

        }

        /* alpha bend for strips */

        App_WrCoCmd_Buffer(phost,COLOR_A(255));
        App_WrCoCmd_Buffer(phost,COLOR_RGB(0xff,0xff,0xff));
        App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
        App_WrCoCmd_Buffer(phost,CLEAR(1,0,0));
        App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
        App_WrCoCmd_Buffer(phost,POINT_SIZE((OuterCircleRadius-2) * 16) );		
        App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));

        App_WrCoCmd_Buffer(phost,BLEND_FUNC(DST_ALPHA,ONE_MINUS_DST_ALPHA));  



        /* Parameters for Strips */
        App_WrCoCmd_Buffer(phost, LINE_WIDTH(2*16));	
        App_WrCoCmd_Buffer(phost,COLOR_RGB(30 ,30 ,30)); 
        Delta = BaseTrackValInit - BaseTrackValSign;

        if((prevtag == 1) && (Read_tag == 1)   &&(Touch!= -32768) )	
        {
            PrevBaseVal = PrevBaseVal - Delta;			
            MoveDeg = (uint16_t)(PrevBaseVal / 182);	

        }

        if(PrevBaseVal > 60294) PrevBaseVal = 60294;
        if(PrevBaseVal <0)PrevBaseVal = 0;
        if(MoveDeg > 330) MoveDeg = 330;
        if(MoveDeg < 30) MoveDeg =31;

        if(MoveDeg <= 90)
        {
            /* First Left edge strip */
            App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));
            angle = 2*da(MoveDeg,-180);
            polarxy((OuterCircleRadius-2),angle,&X1,&Y1,CircleX,CircleY);			
            App_WrCoCmd_Buffer(phost,VERTEX2F(CircleX*16,CircleY*16));		
            App_WrCoCmd_Buffer(phost, VERTEX2F(X1,Y1));
        }

        if(MoveDeg <= 180)
        {
            /* Second Above edge strip */
            App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_A));		
            if(MoveDeg<=90)
            {
                angle = 2*da(90,-180);
                polarxy((OuterCircleRadius-2),angle,&X1,&Y1,CircleX,CircleY);			
            }
            else if (MoveDeg>90)
            {
                angle = 2*da(MoveDeg,-180);
                polarxy((OuterCircleRadius-2),angle,&X1,&Y1,CircleX,CircleY);			
            }
            App_WrCoCmd_Buffer(phost,VERTEX2F(CircleX*16,CircleY*16));		
            App_WrCoCmd_Buffer(phost, VERTEX2F(X1,Y1));

        }
        if(MoveDeg <= 270)
        {
            /* Third Above edge strip */
            App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_R));		
            if(MoveDeg<=180)
            {
                angle = 2*da(180,-180);
                polarxy((OuterCircleRadius-2),angle,&X1,&Y1,CircleX,CircleY);			
            }
            else if (MoveDeg>180)
            {
                angle = 2*da(MoveDeg,-180);
                polarxy((OuterCircleRadius-2),angle,&X1,&Y1,CircleX,CircleY);			
            }	
            App_WrCoCmd_Buffer(phost, VERTEX2F(X1,Y1));
            App_WrCoCmd_Buffer(phost,VERTEX2F(CircleX*16,CircleY*16));					
        }

        if(MoveDeg <= 330)
        {
            /* Fourth Above edge strip */
            App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_B));		
            if(MoveDeg<=270)
            {
                angle = 2*da(270,-180);
                polarxy((OuterCircleRadius-1),angle,&X1,&Y1,CircleX,CircleY);		
            }
            else if (MoveDeg>270)
            {
                angle = 2*da(MoveDeg,-180);
                polarxy((OuterCircleRadius-1),angle,&X1,&Y1,CircleX,CircleY);								
            }
            App_WrCoCmd_Buffer(phost,VERTEX2F((CircleX)*16,CircleY*16));		
            App_WrCoCmd_Buffer(phost, VERTEX2F(X1,Y1));
        }		


        App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE_MINUS_SRC_ALPHA));

        App_WrCoCmd_Buffer(phost, STENCIL_OP(KEEP,KEEP));
        App_WrCoCmd_Buffer(phost, STENCIL_FUNC(ALWAYS,0,255));

        /*alpha blend for rgb bitmap*/
        App_WrCoCmd_Buffer(phost,COLOR_A(0xff));
        App_WrCoCmd_Buffer(phost,COLOR_RGB(0xff,0xff,0xff));
        App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
        App_WrCoCmd_Buffer(phost,CLEAR(1,0,0));
        App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));		
        App_WrCoCmd_Buffer(phost,POINT_SIZE((146/2) * 16) );	
        App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX,CircleY,0,0));
        App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));

        App_WrCoCmd_Buffer(phost,BLEND_FUNC(DST_ALPHA,ONE_MINUS_DST_ALPHA));

        /* Meter Dial bitmap placement */
        App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
        App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX - 146/2,CircleY - 146/2,1,0));
        App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE_MINUS_SRC_ALPHA));
        App_WrCoCmd_Buffer(phost,END());

        App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
        App_WrCoCmd_Buffer(phost,COLOR_A(20));
        App_WrCoCmd_Buffer(phost,TAG(1));

        /* Alpha Meter Dial bitmap placement & rotation */
        App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
        Gpu_CoCmd_LoadIdentity(phost);
        Gpu_CoCmd_Translate(phost,F16(73),F16(73));
        Gpu_CoCmd_Rotate(phost,MoveDeg*65536/360);
        Gpu_CoCmd_Translate(phost,F16(-(73)),F16(-(73)));		  
        Gpu_CoCmd_SetMatrix(phost);	
        App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX - 146/2,CircleY - 146/2,0,0));		
        App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

        /* restore default alpha value */
        App_WrCoCmd_Buffer(phost,COLOR_A(255));		

        Gpu_CoCmd_FgColor(phost, 0x59D3E8);
        Gpu_CoCmd_GradColor(phost, 0x7BF8FA);

        Gpu_CoCmd_Button(phost, (ScreenWidthMid - 40), ButY, 80, ButWid, 28, 0, "");// placement of button

        App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));

        App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));//color of number(text) color



        if((MoveDeg >= 30)&& (MoveDeg <= 330))
        {			
            Volume = PrevBaseVal>>8;						
            App_WrCoCmd_Buffer(phost,BEGIN(LINES));
            App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
            App_WrCoCmd_Buffer(phost,COLOR_RGB(30 ,30 ,30));
            angle = da(MoveDeg,-180)*2;
            polarxy((OuterCircleRadius-3),angle,&X1,&Y1,CircleX,CircleY);	
            polarxy(74,angle,&X2,&Y2,CircleX,CircleY);	
            App_WrCoCmd_Buffer(phost, VERTEX2F(X1,Y1));
            App_WrCoCmd_Buffer(phost, VERTEX2F(X2,Y2));						
        }

        Gpu_CoCmd_Number(phost,(ScreenWidthMid),NoY,28,OPT_CENTER,Volume);
        Gpu_Hal_Wr8(phost, REG_VOL_SOUND,Volume);
        App_WrCoCmd_Buffer(phost,DISPLAY());

        Gpu_CoCmd_Swap(phost);

        /* Download the commands into fifo */
        App_Flush_Co_Buffer(phost);

        /* Wait till coprocessor completes the operation */
        Gpu_Hal_WaitCmdfifo_empty(phost);
        prevtag = Read_tag;		
        BaseTrackValInit = BaseTrackValSign;
        Gpu_Hal_Sleep(30);
    }while(1);


}

#endif

#ifdef Relative_Dial


void Relative_Meter_Dial()
{
    uint8_t i=0,j=0,k=0,Read_tag,prevtag=0,currtag;
    int16_t ScreenWidthMid,ScreenHeightMid,NextTh=0,Touch,drag=0;
    int16_t OuterCircleRadius,PrevValDeg=0,CircleX,CircleY,ButY,ButWid,NoY,Delta,G1,G2,Sx1,Sy1;

    int32_t PrevTh=0,CurrTh,adjusting=0,BaseTrackValSign = 0,Vx1=0,Vy1=0,Vx2=0,Vy2=0,Volume=0,PrevBaseTrackVal=0,X2,Y2;

    uint32_t RotaryTag=0;	

    int32_t change;
    int32_t sx,storesx,storepresx;	


    ScreenWidthMid = DispWidth/2;
    ScreenHeightMid = DispHeight/2;

    PrevValDeg =0;
    Sx1 = Sy1 = 180;



#ifdef DISPLAY_RESOLUTION_WQVGA
    Sx1 = Sy1 = 240;
    OuterCircleRadius = ScreenWidthMid - 144;
    CircleX = ScreenWidthMid;
    CircleY = ScreenHeightMid;
    ButY = DispHeight -262;
    ButWid = 30;
    NoY = (DispHeight-262 + 30/2);
#endif
#if defined(DISPLAY_RESOLUTION_QVGA) || defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
    Sx1 = Sy1 = 130;
    ButY = 1;
    ButWid = 23;
    NoY = 12;
    OuterCircleRadius = ScreenWidthMid - 64;
    CircleX = ScreenWidthMid-70 + 146/2;
    CircleY = ScreenHeightMid-67 + 146/2;

#endif

#ifdef DISPLAY_RESOLUTION_WVGA
    Sx1 = Sy1 = 400;
    OuterCircleRadius = ScreenWidthMid - 100;
    CircleX = ScreenWidthMid;
    CircleY = ScreenHeightMid;
    ButY = DispHeight -460;
    ButWid = 150;
    NoY = (DispHeight-460 + 30/2);
#endif



    Gpu_Hal_WrCmd32(phost, CMD_MEMSET);
    Gpu_Hal_WrCmd32(phost, 0L);//starting address of memset
    Gpu_Hal_WrCmd32(phost, 55L);//value of memset
    Gpu_Hal_WrCmd32(phost,256*1024);



    /* set the vol to MAX */
    Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0);
    /* Play carousel*/
    Gpu_Hal_Wr16(phost, REG_SOUND, 8);

    /* Construction of starting screen shot, assign all the bitmap handles here */
    App_WrCoCmd_Buffer(phost,CMD_DLSTART);
    App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));

    App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

    /* Download the raw data into intended locations */
    {
        App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));			
        Load_RawDataFromfile(SAMAPP_Relative_Meter_Dial_rgb,SAMAPP_Relative_MeterDial_Size146x146_rgb,1,RGB565,146*2,146,146);	

        Load_RawDataFromfile(SAMAPP_Abs_Base_Dial_L1,SAMAPP_Abs_BaseDial_Size146x146_L1,0,L4,146,146,146);
    }

    App_WrCoCmd_Buffer(phost,DISPLAY());
    Gpu_CoCmd_Swap(phost);

    /* Download the commands into fifo */
    App_Flush_Co_Buffer(phost);

    /* Wait till coprocessor completes the operation */
    Gpu_Hal_WaitCmdfifo_empty(phost);


    /*asiign track to the bitmap*/
    Gpu_CoCmd_Track(phost,(CircleX)+40,CircleY, DIAL, DIAL, 1);

    Gpu_Hal_Wr8(phost, REG_PLAY,1);

    do{
        App_WrCoCmd_Buffer(phost,CLEAR_TAG(255));
        Read_tag = Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
        Touch = Gpu_Hal_Rd16(phost, REG_TOUCH_SCREEN_XY+2);
        RotaryTag = Gpu_Hal_Rd32(phost,REG_TRACKER);
        Gpu_CoCmd_Dlstart(phost);
        App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
        App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));

        App_WrCoCmd_Buffer(phost, SCISSOR_XY(0,0) );

#if defined(DISPLAY_RESOLUTION_WQVGA) 
        Gpu_CoCmd_Gradient(phost,0, DispHeight, 0x1E1E1E,ScreenWidthMid-60/*180*/, ScreenHeightMid-21/*115*/, 0x999999);

        App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
        App_WrCoCmd_Buffer(phost,BEGIN(LINES));

        App_WrCoCmd_Buffer(phost,VERTEX2F((DispWidth-2)*16,(DispHeight)*16 ));
        App_WrCoCmd_Buffer(phost,VERTEX2F(ScreenWidthMid*16,0 *16));

#elif defined(DISPLAY_RESOLUTION_QVGA)
        Gpu_CoCmd_Gradient(phost,0, DispHeight, 0x1E1E1E,140/*120*/, 140/*101*/, 0x999999);

        App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
        App_WrCoCmd_Buffer(phost,BEGIN(LINES));

        App_WrCoCmd_Buffer(phost,VERTEX2F((DispWidth-2)*16,(DispHeight)*16 ));
        App_WrCoCmd_Buffer(phost,VERTEX2F(146*16,0 *16));
#elif defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
        Gpu_CoCmd_Gradient(phost,0, DispHeight, 0x1E1E1E,ScreenWidthMid-60, ScreenHeightMid-21, 0x999999);

        App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));

        App_WrCoCmd_Buffer(phost,BEGIN(LINES));

        App_WrCoCmd_Buffer(phost,VERTEX2F((0)*16,(30)*16 ));
        App_WrCoCmd_Buffer(phost,VERTEX2F(DispWidth*16,154 *16));
#elif defined(DISPLAY_RESOLUTION_WVGA)
        Gpu_CoCmd_Gradient(phost,0, DispHeight, 0x1E1E1E,300/*60*/, 217/*21*/, 0x999999);

        App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
        App_WrCoCmd_Buffer(phost,BEGIN(LINES));

        App_WrCoCmd_Buffer(phost,VERTEX2F((DispWidth-2)*16,(DispHeight)*16 ));
        App_WrCoCmd_Buffer(phost,VERTEX2F(ScreenWidthMid*16,0 *16));
#endif




        sx = 0;	 
        currtag = (RotaryTag&0xff);
        if(1 == currtag)
        {
            sx = RotaryTag >> 16;
        }


        if((prevtag == 1) && (1 == currtag))
        {
            scroller.vel = (-scroller.dragprev + sx);

            if((scroller.vel > 32000))
            {
                scroller.vel-= 65536;
            }

            else if (scroller.vel < -32768)
            {
                scroller.vel+= 65536;
            }
            scroller.vel <<= 4;

            storepresx = scroller.dragprev;
            storesx = sx;			
        } 
        else
        {
            change = MAX(1, abs(scroller.vel) >> 5);
            if (scroller.vel < 0)
            scroller.vel += change;

            if (scroller.vel > 0)
            scroller.vel -= change;

        }
        prevtag = currtag;
        scroller.dragprev = sx;
        scroller.base += scroller.vel;


        drag = scroller.base>>4;


        /*Managing the track*/	
        {
            if((1 == currtag))
            {
                CurrTh = sx;				

                if((adjusting!=0) )
                {

                    PrevBaseTrackVal = (CurrTh - PrevTh);

                    if((PrevBaseTrackVal > 32000))
                    {
                        PrevBaseTrackVal-= 65536;
                    }
                    else if (PrevBaseTrackVal < -32768)
                    {
                        PrevBaseTrackVal+= 65536;
                    }
                    BaseTrackValSign += PrevBaseTrackVal;
                    BaseTrackVal += PrevBaseTrackVal;

                }

                adjusting = (RotaryTag & 0xff);
                PrevTh = CurrTh;
                NextTh = (uint16_t)(BaseTrackValSign / 182);			
                PrevValDeg = NextTh;

            }

            else if((0 != scroller.vel))
            {

                BaseTrackValSign += (scroller.vel>>4); //original copy
                BaseTrackVal +=  (scroller.vel>>4);;
                NextTh = (uint16_t)(BaseTrackValSign / 182);			
                PrevValDeg = NextTh;
                adjusting = 0;
            }
            else
            {
                adjusting = 0;
            }

        }

        /*alpha blend for rgb bitmap*/

        App_WrCoCmd_Buffer(phost,COLOR_A(0xff));
        App_WrCoCmd_Buffer(phost,COLOR_RGB(0xff,0xff,0xff));
        App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
        App_WrCoCmd_Buffer(phost,CLEAR(1,0,0));
        App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));		
        App_WrCoCmd_Buffer(phost,POINT_SIZE((146/2) * 16) );	
        App_WrCoCmd_Buffer(phost,VERTEX2II(CircleX +40,CircleY,0,0));
        App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));

        App_WrCoCmd_Buffer(phost,BLEND_FUNC(DST_ALPHA,ONE_MINUS_DST_ALPHA));

        /* Meter Dial bitmap placement */
        App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));


        App_WrCoCmd_Buffer(phost,VERTEX2II((CircleX - 146/2)+40,CircleY - 146/2,1,0));

        App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE_MINUS_SRC_ALPHA));


        Delta = BaseTrackValSign - BaseTrackValInit;


        App_WrCoCmd_Buffer(phost,COLOR_A(20));

        /* Alpha Meter Dial bitmap placement & rotation */
        App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
        App_WrCoCmd_Buffer(phost,TAG_MASK(1));

        Gpu_CoCmd_LoadIdentity(phost);
        Gpu_CoCmd_Translate(phost,F16(73),F16(73));
        Gpu_CoCmd_Rotate(phost,PrevValDeg*65536/360);
        Gpu_CoCmd_Translate(phost,F16(-(73)),F16(-(73)));		  
        Gpu_CoCmd_SetMatrix(phost);	
        App_WrCoCmd_Buffer(phost,TAG(1));
        App_WrCoCmd_Buffer(phost,VERTEX2II((CircleX - 146/2)+40,CircleY - 146/2,0,0));			


        App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
        App_WrCoCmd_Buffer(phost,TAG(1));	
        /* Draw points around the bitmap*/
        App_WrCoCmd_Buffer(phost,COLOR_A(255));	
        App_WrCoCmd_Buffer(phost, BEGIN(FTPOINTS));		
        App_WrCoCmd_Buffer(phost,POINT_SIZE(5*16));
        App_WrCoCmd_Buffer(phost,COLOR_RGB(227 ,249 ,20)); 
        angle = da(PrevValDeg,-180)*2;
        polarxy(55,angle,&X2,&Y2,CircleX+40,CircleY);

        App_WrCoCmd_Buffer(phost, VERTEX2F(X2,Y2));		
        //printf("%d\n,Touch, %d\n,prevtag", prevtag,Touch);
        App_WrCoCmd_Buffer(phost,TAG_MASK(0));

        {
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
            uint8_t statusButtonH = 20, statusButtonW = 30, buttonsPerRow = 6, rowGap = 10, curRow = 0;
            uint16_t edgeOffset = 30, startingX = 10 , startingY = DispHeight - statusButtonH - edgeOffset; 

            uint16_t buttonVerticalGap = (DispHeight - edgeOffset * 2 - buttonsPerRow * statusButtonH) / (buttonsPerRow - 1);

            Vy1 = startingY;
#endif

            for(k=0;k<=11;k++)
            {
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
                Vx1 = startingX + (curRow * statusButtonW) + (curRow * rowGap);

#else
                Vy1 = 200;			
                if(k==0)/* First button Vertices*/
                {
                    Vx1 = 10;Vy1 = 200;
                }
                else if((k== 1)||(k==2)||(k==3))/* Second,third,fourth button Vertices*/
                {
                    Vx1 = 10;Vy1 = Vy1-(k)*60;
                }
                else if(k==4)/* fifth button Vertices*/
                {
                    Vx1 = 40;Vy1 = 200;
                }
                else if((k== 5)|(k==6)|(k==7))/* sixth,seventh,eighth button Vertices*/
                {
                    Vx1 =40;Vy1 = Vy1-(k-4)*60 ;
                }
                else if(k==8)/* ninth button Vertices*/
                {
                    Vx1 = 70;Vy1 = 200;
                }
                else if((k== 9)|(k==10)|(k==11))/* tenth,eleven,twelth button Vertices*/
                {
                    Vx1 =70;Vy1 = Vy1-(k-8)*60 ;
                }
#endif			

                if(BaseTrackVal < 11*65535)
                {
                    /* Rotary tag when within range of particular index, highlight only the indexed button in yellow*/
                    if(((BaseTrackVal >= k*65535)&&(BaseTrackVal < (k+1)*65535)))//|| ((BaseTrackValSign<0)&&(k==0)))
                    {
                        Gpu_CoCmd_FgColor(phost,0xE3F914);//yellow
                        Gpu_CoCmd_Button(phost,Vx1,Vy1,20,30,16,0,"");	
                    }
                    else/* Color the non indexed buttons in grey color*/
                    {
                        Gpu_CoCmd_FgColor(phost,0x787878);//grey
                        Gpu_CoCmd_Button(phost,Vx1,Vy1,20,30,16,0,"");
                    }
                }

                /*Limit track value when its greater than 12th button(11th index) and also highlight twelth button*/
                else if((BaseTrackVal >= 11*65535))
                {
                    BaseTrackVal = 11*65535;

                    if(k<11)
                    {
                        Gpu_CoCmd_FgColor(phost,0x787878);//grey
                    }
                    else	
                    {
                        Gpu_CoCmd_FgColor(phost,0xE3F914);//yellow					
                    }
                    Gpu_CoCmd_Button(phost,Vx1,Vy1,20,30,16,0,"");
                }

                /*Limit track value when its negative and also highlight first button*/
                if((BaseTrackVal<0)&&(k==0))
                {
                    if((Delta>0))
                    BaseTrackVal =0;
                    Gpu_CoCmd_FgColor(phost,0xE3F914);//yellow	
                    Gpu_CoCmd_Button(phost,Vx1,Vy1,20,30,16,0,"");
                }

#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
                Vy1 -= (statusButtonH + buttonVerticalGap);

                if(Vy1 < edgeOffset){
                    curRow++;
                    Vy1 = startingY;
                }
#endif
            }

        }

        /* play volume*/		

        if(BaseTrackVal < 0)Volume =0;
        else
        Volume = BaseTrackVal>>12;

        Gpu_Hal_Wr8(phost, REG_VOL_SOUND,Volume);


        App_WrCoCmd_Buffer(phost,DISPLAY());
        Gpu_CoCmd_Swap(phost);
        /* Download the commands into fifo */
        App_Flush_Co_Buffer(phost);
        /* Wait till coprocessor completes the operation */
        Gpu_Hal_WaitCmdfifo_empty(phost);
        BaseTrackValInit = BaseTrackValSign;
        prevtag = Read_tag;
        Gpu_Hal_Sleep(30);
    }while(1);



}
#endif


/* Startup Screen Information*/ 
#if defined(ARDUINO_PLATFORM)
prog_char8_t * const info[] =
#else
char *info[] =
#endif 
{ "EVE Meter Dial Application",
    "APP to demonstrate interactive Meter dial,",
    "using Points, Track",
    "& Stencil"
}; 

#if defined (MSVC_PLATFORM) || defined (FT9XX_PLATFORM)
/* Main entry point */
int32_t main(int32_t argc,char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM) || defined(MSVC_FT800EMU)
void setup()
#endif
{
    phost = &host;
    /* Init HW Hal */
    App_Common_Init(phost);
    /* Show Logo, do calibration and display welcome screeen */
    App_Common_Start(phost,info);
    /* Main application */
#ifdef Absolute_Dial
    Abs_Meter_Dial();
#endif
#ifdef Relative_Dial
    Relative_Meter_Dial();
#endif /*Relative_Dial*/
    /* Close all the opened handles */
    App_Common_Close(phost);

    /* Close all the opened handles */
    Gpu_Hal_Close(phost);
    Gpu_Hal_DeInit();
#if defined (MSVC_PLATFORM) || defined (FT9XX_PLATFORM)
    return 0;
#endif
}

void loop()
{
}



/* Nothing beyond this */



